@@ -34,7 +34,7 @@ import dectate
 from reg import methodify
 
 from .authentication import Identity, NoIdentity
-from .view import render_view, render_json, render_html, ViewRegistry
+from .view import render_view, render_json, render_html, View
 from .traject import Path
 from .converter import ConverterRegistry
 from .tween import TweenRegistry
@@ -170,32 +170,30 @@ class PredicateAction(dectate.Action):
 
     def __init__(self, dispatch, name, default, index,
                  before=None, after=None):
-        """Register custom predicate for a predicate_dispatch function.
+        """Register a custom predicate for a dispatch method.
 
-        The function registered should have arguments that are the
-        same as the arguments of the :reg:`reg.predicate_dispatch`
-        function. From these arguments it should determine a predicate
-        value and return it. The predicates for a ``predicate_dispatch``
-        function are ordered by their before and after arguments.
+        The function to be registered should have the same arguments
+        as the dispatch method and return a value that is used when
+        registering an implementation for the dispatch method.
 
-        You can then register a function to dispatch to using the
-        :meth:`App.method` directive. This takes the
-        ``predicate_dispatch`` or ``dispatch`` function as its first
-        argument and the predicate key to match on as its other
+        The predicates are ordered by their before and after
         arguments.
 
-        :param dispatch: the dispatch function this predicate
-           is for.
-        :param name: the name of the predicate. This is used when
-          constructing a predicate key from a predicate dictionary.
-        :param default: the default value for a predicate, in case the
-          value is missing in the predicate dictionary.
-        :param index: the index to use. Typically :class:`reg.KeyIndex` or
-          :class:`reg.ClassIndex`.
+        :param dispatch: the dispatch method this predicate is for.
+          You can use the :meth:`App.method` directive to add a
+          dispatch method to an app.
+        :param name: the name used to identify the predicate when
+          registering the implementation of the dispatch method.
+        :param default: the expected value of the predicate, to be used
+          when registering an implementation if the expected value for
+          the predicate is not given explicitly.
+        :param index: the index to use. Typically
+          :class:`reg.KeyIndex` or :class:`reg.ClassIndex`.
         :param before: predicate function this function wants to have
-           priority over.
+          priority over.
         :param after: predicate function we want to have priority over
-           this one.
+          this one.
+
         """
         self.dispatch = dispatch
         self.name = name
@@ -243,7 +241,7 @@ class MethodAction(dectate.Action):
         override parts of the Morepath framework itself.
 
         The ``dispatch_method`` argument is a dispatch method, so a
-        method on a :class:`morepath.App`` class marked with
+        method on a :class:`morepath.App` class marked with
         :func:`reg.dispatch_method`, so for instance ``App.foo``. The
         registered function gets the instance of this app class as its
         first argument. The registered function must have the same arguments
@@ -265,7 +263,7 @@ class MethodAction(dectate.Action):
 
     def identifier(self, app_class):
         return (self.dispatch_method,
-                self.dispatch_method.key_dict_to_predicate_key(self.key_dict))
+                self.dispatch_method.by_predicates(**self.key_dict).key)
 
     def perform(self, obj, app_class):
         getattr(app_class, self.dispatch_method.__name__).register(
@@ -339,7 +337,7 @@ class PathAction(dectate.Action):
         path_registry.register_path(
             self.model, self.path,
             self.variables, self.converters, self.required,
-            self.get_converters, self.absorb,
+            self.get_converters, self.absorb, self.code_info,
             obj)
 
 
@@ -446,7 +444,7 @@ class PermissionRuleAction(dectate.Action):
         """Declare whether a model has a permission.
 
         The decorated function receives ``app``, ``model``,
-        `permission`` (instance of any permission object) and
+        ``permission`` (instance of any permission object) and
         ``identity`` (:class:`morepath.Identity`) parameters. The
         ``app`` argument is optional. The decorated function should
         return ``True`` only if the given identity exists and has that
@@ -619,7 +617,7 @@ def isbaseclass_notfound(a, b):
 
 class ViewAction(dectate.Action):
     config = {
-        'view_registry': ViewRegistry,
+        'template_engine_registry': TemplateEngineRegistry,
     }
 
     depends = [SettingAction, PredicateAction,
@@ -642,6 +640,8 @@ class ViewAction(dectate.Action):
         'body_model': isbaseclass_notfound,
     }
 
+    app_class_arg = True
+
     def __init__(self, model, render=None, template=None,
                  permission=None,
                  internal=False, **predicates):
@@ -703,18 +703,22 @@ class ViewAction(dectate.Action):
         self.predicates = predicates
 
     def key_dict(self):
+        """Return a dict containing view registration info,
+        for instance model, request_method, etc."""
         result = self.predicates.copy()
         result['model'] = self.model
         return result
 
-    def identifier(self, view_registry):
-        return view_registry.predicate_key(self.key_dict())
+    def identifier(self, template_engine_registry, app_class):
+        return app_class.get_view.by_predicates(**self.key_dict()).key
 
-    def perform(self, obj, view_registry):
-        view_registry.register_view(
-            self.key_dict(), obj,
-            self.render, self.template,
-            self.permission, self.internal)
+    def perform(self, obj, template_engine_registry, app_class):
+        render = self.render
+        if self.template is not None:
+            render = template_engine_registry.get_template_render(
+                self.template, render)
+        v = View(obj, render, self.permission, self.internal, self.code_info)
+        app_class.get_view.register(v, **self.key_dict())
 
 
 class JsonAction(ViewAction):
@@ -876,7 +880,7 @@ class MountAction(PathAction):
         path_registry.register_mount(
             self.app, self.path, self.variables,
             self.converters, self.required,
-            self.get_converters, self.name, obj)
+            self.get_converters, self.name, self.code_info, obj)
 
 
 class DeferLinksAction(dectate.Action):
